@@ -343,12 +343,47 @@ struct xsimd_api_integral_types_functions
 {
     using value_type = typename scalar_type<T>::type;
 
+    void test_bitwise_lshift()
+    {
+        value_type val0(12);
+        value_type val1(3);
+        value_type r = val0 << val1;
+        CHECK_EQ(extract(xsimd::bitwise_lshift(T(val0), T(val1))), r);
+    }
+
+    void test_bitwise_rshift()
+    {
+        value_type val0(12);
+        value_type val1(3);
+        value_type r = val0 >> val1;
+        CHECK_EQ(extract(xsimd::bitwise_rshift(T(val0), T(val1))), r);
+    }
+
     void test_mod()
     {
         value_type val0(5);
         value_type val1(3);
         CHECK_EQ(extract(xsimd::mod(T(val0), T(val1))), val0 % val1);
     }
+
+    void test_rotl()
+    {
+        constexpr auto N = std::numeric_limits<value_type>::digits;
+        value_type val0(12);
+        value_type val1(3);
+        value_type r = (val0 << val1) | (val0 >> (N - val1));
+        CHECK_EQ(extract(xsimd::rotl(T(val0), T(val1))), r);
+    }
+
+    void test_rotr()
+    {
+        constexpr auto N = std::numeric_limits<value_type>::digits;
+        value_type val0(12);
+        value_type val1(3);
+        value_type r = (val0 >> val1) | (val0 << (N - val1));
+        CHECK_EQ(extract(xsimd::rotr(T(val0), T(val1))), r);
+    }
+
     void test_sadd()
     {
         value_type val0(122);
@@ -367,11 +402,31 @@ TEST_CASE_TEMPLATE("[xsimd api | integral types functions]", B, INTEGRAL_TYPES)
 {
     xsimd_api_integral_types_functions<B> Test;
 
+    SUBCASE("bitwise_lshift")
+    {
+        Test.test_bitwise_lshift();
+    }
+
+    SUBCASE("bitwise_rshift")
+    {
+        Test.test_bitwise_rshift();
+    }
+
     SUBCASE("mod")
     {
         Test.test_mod();
     }
 
+    SUBCASE("rotl")
+    {
+        Test.test_rotl();
+    }
+
+    SUBCASE("rotr")
+    {
+        Test.test_rotr();
+    }
+
     SUBCASE("sadd")
     {
         Test.test_sadd();
@@ -463,7 +518,11 @@ struct xsimd_api_float_types_functions
     void test_exp10()
     {
         value_type val(2);
+#ifdef EMSCRIPTEN
+        CHECK_EQ(extract(xsimd::exp10(T(val))), doctest::Approx(std::pow(value_type(10), val)));
+#else
         CHECK_EQ(extract(xsimd::exp10(T(val))), std::pow(value_type(10), val));
+#endif
     }
     void test_exp2()
     {
@@ -483,8 +542,11 @@ struct xsimd_api_float_types_functions
     void test_erfc()
     {
         // FIXME: can we do better?
-        value_type val(0);
-        CHECK_EQ(extract(xsimd::erfc(T(val))), doctest::Approx(std::erfc(val)).epsilon(10e-7));
+        for (float f : { 0.f, 0.1f, 0.2f, 0.3f, 0.4f, 0.5f, 0.6f, 0.7f, 0.8f, 0.9f })
+        {
+            value_type val(f);
+            CHECK_EQ(extract(xsimd::erfc(T(val))), doctest::Approx(std::erfc(val)).epsilon(10e-8));
+        }
     }
     void test_fabs()
     {
@@ -549,21 +611,6 @@ struct xsimd_api_float_types_functions
         value_type val(4);
         CHECK_EQ(extract(xsimd::is_odd(T(val))), (val == long(val)) && (long(val) % 2 == 1));
     }
-    void test_isinf()
-    {
-        value_type val(4);
-        CHECK_EQ(extract(xsimd::isinf(T(val))), std::isinf(val));
-    }
-    void test_isfinite()
-    {
-        value_type val(4);
-        CHECK_EQ(extract(xsimd::isfinite(T(val))), std::isfinite(val));
-    }
-    void test_isnan()
-    {
-        value_type val(4);
-        CHECK_EQ(extract(xsimd::isnan(T(val))), std::isnan(val));
-    }
     void test_ldexp()
     {
         value_type val0(4);
@@ -618,7 +665,12 @@ struct xsimd_api_float_types_functions
     {
         value_type val0(3);
         value_type val1(4);
+#ifndef EMSCRIPTEN
         CHECK_EQ(extract(xsimd::polar(T(val0), T(val1))), std::polar(val0, val1));
+#else
+        CHECK_EQ(std::real(extract(xsimd::polar(T(val0), T(val1)))), doctest::Approx(std::real(std::polar(val0, val1))));
+        CHECK_EQ(std::imag(extract(xsimd::polar(T(val0), T(val1)))), doctest::Approx(std::imag(std::polar(val0, val1))));
+#endif
     }
     void test_pow()
     {
@@ -835,18 +887,6 @@ TEST_CASE_TEMPLATE("[xsimd api | float types functions]", B, FLOAT_TYPES)
     {
         Test.test_is_odd();
     }
-    SUBCASE("isinf")
-    {
-        Test.test_isinf();
-    }
-    SUBCASE("isfinite")
-    {
-        Test.test_isfinite();
-    }
-    SUBCASE("isnan")
-    {
-        Test.test_isnan();
-    }
     SUBCASE("ldexp")
     {
         Test.test_ldexp();
@@ -994,6 +1034,24 @@ struct xsimd_api_complex_types_functions
         value_type val(1);
         CHECK_EQ(extract(xsimd::proj(T(val))), std::proj(val));
     }
+
+    void test_isinf()
+    {
+        value_type val(4);
+        CHECK_EQ(extract(xsimd::isinf(T(val))), std::isinf(std::real(val)));
+    }
+
+    void test_isfinite()
+    {
+        value_type val(4);
+        CHECK_EQ(extract(xsimd::isfinite(T(val))), std::isfinite(std::real(val)));
+    }
+
+    void test_isnan()
+    {
+        value_type val(4);
+        CHECK_EQ(extract(xsimd::isnan(T(val))), std::isnan(std::real(val)));
+    }
 };
 
 TEST_CASE_TEMPLATE("[xsimd api | complex types functions]", B, COMPLEX_TYPES)
@@ -1018,6 +1076,21 @@ TEST_CASE_TEMPLATE("[xsimd api | complex types functions]", B, COMPLEX_TYPES)
     {
         Test.test_proj();
     }
+
+    SUBCASE("isinf")
+    {
+        Test.test_isinf();
+    }
+
+    SUBCASE("isfinite")
+    {
+        Test.test_isfinite();
+    }
+
+    SUBCASE("isnan")
+    {
+        Test.test_isnan();
+    }
 }
 
 /*
@@ -1083,6 +1156,18 @@ struct xsimd_api_all_types_functions
         CHECK_EQ(extract(xsimd::add(T(val0), T(val1))), val0 + val1);
     }
 
+    void test_decr()
+    {
+        value_type val0(1);
+        CHECK_EQ(extract(xsimd::decr(T(val0))), val0 - value_type(1));
+    }
+
+    void test_decr_if()
+    {
+        value_type val0(1);
+        CHECK_EQ(extract(xsimd::decr_if(T(val0), T(val0) != T(0))), val0 - value_type(1));
+    }
+
     void test_div()
     {
         value_type val0(1);
@@ -1121,6 +1206,18 @@ struct xsimd_api_all_types_functions
         CHECK_EQ(extract(xsimd::fnma(T(val0), T(val1), T(val2))), -(val0 * val1) + val2);
     }
 
+    void test_incr()
+    {
+        value_type val0(1);
+        CHECK_EQ(extract(xsimd::incr(T(val0))), val0 + value_type(1));
+    }
+
+    void test_incr_if()
+    {
+        value_type val0(1);
+        CHECK_EQ(extract(xsimd::incr_if(T(val0), T(val0) != T(0))), val0 + value_type(1));
+    }
+
     void test_mul()
     {
         value_type val0(2);
@@ -1161,6 +1258,12 @@ TEST_CASE_TEMPLATE("[xsimd api | all types functions]", B, ALL_TYPES)
         Test.test_add();
     }
 
+    SUBCASE("decr")
+    {
+        Test.test_decr();
+        Test.test_decr_if();
+    }
+
     SUBCASE("div")
     {
         Test.test_div();
@@ -1186,6 +1289,12 @@ TEST_CASE_TEMPLATE("[xsimd api | all types functions]", B, ALL_TYPES)
         Test.test_fnma();
     }
 
+    SUBCASE("incr")
+    {
+        Test.test_incr();
+        Test.test_incr_if();
+    }
+
     SUBCASE("mul")
     {
         Test.test_mul();
@@ -1231,3 +1340,38 @@ TEST_CASE_TEMPLATE("[xsimd api | all floating point types functions]", B, ALL_FL
     xsimd_api_all_floating_point_types_functions<B> Test;
     Test.test_neq_nan();
 }
+
+/*
+ * Functions that apply only to mask type
+ */
+template <typename T>
+struct xsimd_api_all_mask_functions
+{
+    using value_type = typename scalar_type<T>::type;
+
+    void test_all()
+    {
+        value_type val(1);
+        CHECK_EQ(xsimd::all(T(val) == T(val)), xsimd::all(val == val));
+    }
+
+    void test_any()
+    {
+        value_type val(1);
+        CHECK_EQ(xsimd::any(T(val) == T(val)), xsimd::any(val == val));
+    }
+
+    void test_none()
+    {
+        value_type val(1);
+        CHECK_EQ(xsimd::none(T(val) != T(val)), xsimd::none(val != val));
+    }
+};
+
+TEST_CASE_TEMPLATE("[xsimd api | all mask functions]", B, ALL_TYPES)
+{
+    xsimd_api_all_mask_functions<B> Test;
+    Test.test_all();
+    Test.test_any();
+    Test.test_none();
+}
